/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file c_util.c
 * \brief useful c routines
 */

#include "c_util.h"
 
/*!
 * \brief Split a string (str) given a char separator (sep) and fill a table (str_split)
 * 
 * ex : hello_world__!! with '_' as a separator --> return 3 && str_split : \n
 * 0 : hello \n
 * 1 : world \n
 * 2 : !! \n
 * If no 'sep' is found in 'str' then 0 is returned and str_split is set to NULL \n
 * 
 * \warning always test return value with a >
 */
int string_split(char *str, char ***str_split, char sep)
{
	unsigned int i=0, j=0;
	ptrdiff_t diff=0;
	char *sep_pos;
	
	while( str[i] && (str[i] != '\n') && (str[i] != '\r'))
	{
		if( (str[i] == sep) && (str[(i-1)] != sep))
			j++;
		i++;
	}

	if(j==0)
	{
		str_split = NULL;
		return 0;
	}

	*str_split = (char **)malloc(sizeof(char *)*(j+1));
	i = 0;
	
	/* while( strchr(str, sep) != NULL) */
	while( (sep_pos = strchr(str, sep)) )
	{
		diff = sep_pos - str;
		if(diff == 0)
		{	
			str++;
			continue;
		}
		
		(*str_split)[i] = (char *)malloc(sizeof(char)*(diff+1));
		strncpy((*str_split)[i], str, diff);
		(*str_split)[i][diff] = '\0';
		str += (diff+1);
		i++;	
	}

	diff = strchr(str, '\0') - str;
	
	(*str_split)[i] = (char *)malloc(sizeof(char)*(diff+1)); 
	strncpy((*str_split)[i], str, diff);
	(*str_split)[i][diff] = '\0';
	
	return (j+1);
	/* return j; */
}

/*!
 * \brief free an array
 *
 * use as follow : array_free(i, (void **)array)
 */
void array_free(int num, void **array)
{
	int i;
	if(array != NULL)
	{
		for(i=0; i<num;i++)
		{
			if(array[i] != NULL)
				free(array[i]);
		}
		free(array);
	}
}

/*! \brief replace first '\n' or '\r' in str with '\0' */
void str_eol(char *str)
{
	char *ch_pos = NULL;
	ptrdiff_t diff=0;

	ch_pos	= strchr(str, '\r');
	if( ch_pos != NULL )
	{
		diff = ch_pos-str;
		str[ (int)diff ]='\0';
	}
	else
	{
		ch_pos = strchr(str, '\n');
		if( ch_pos != NULL)
		{
			diff = ch_pos-str;
			str[ (int)diff ] = '\0';
		}
	}	
}

/*! \brief simple fopen encapsulation */
FILE *file_open(const char *filename, const char *mode)
{
	FILE *fp = NULL;
	
	if( strlen(filename) > FILENAME_MAX )
		return NULL;
	
	if( !(fp = fopen(filename, mode)) )
		return NULL;

	return fp;
}

/*! \brief compare 2 float values (within #HKW_EPSILON)
 *
 * return SDL_TRUE if f2 == f1 else return SDL_FALSE
 */
SDL_bool float_equal( GLfloat f1, GLfloat f2 )
{
	if( (f1 < (f2+HKW_EPSILON)) && (f1 > (f2-HKW_EPSILON)) )
		return SDL_TRUE;
	else
		return SDL_FALSE;
}






