/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file camera.c
 * \brief camera functions */

#include "camera.h"
#include "ingame.h"

/*! \brief Init a camera with default values */
void camera_init(camera_t *cam)
{
	quat_init_id(&cam->rotation);

	cam->angle[0] = 0.0f;
	cam->angle[1] = 0.0f;
	cam->angle[2] = 0.0f;
	cam->position.x = 0.0f;
	cam->position.y = 0.0f;
	cam->position.z = -6.0f;

	/* to be removed ? */
	cam->motion = 0.0f;
}

/*! \brief Rotate a camera with the mouse */
void camera_mouse(camera_t *cam)
{
	int cur_x, cur_y;
	int mid_x, mid_y;
	GLfloat angle_x;
	GLfloat angle_y;
	quat_t qx, qy;

	SDL_Surface *screen = SDL_GetVideoSurface();
	mid_x = screen->w >> 1, mid_y = screen->h >> 1;
	
	/* get mouse position */
	SDL_GetMouseState(&cur_x, &cur_y);

	/* return if the mouse was not moved */
	if( (cur_x == mid_x) && (cur_y == mid_y) )
		return;

	SDL_WarpMouse(mid_x, mid_y);

	/* update angle of rotations */
	angle_y = (float)( (mid_x - cur_x) ) / (float)10.0f;
	angle_x = (float)( (mid_y - cur_y) ) / (float)10.0f;

	cam->angle[1] += angle_y;
	cam->angle[0] += angle_x;
	
	if( cam->angle[1] > 360.0f )
		cam->angle[1] -= 360.0f;
	else if(cam->angle[1] < -360.0f )
		cam->angle[1] += 360.0f;
	
	if( cam->angle[0] > 360.0f )
		cam->angle[0] -= 360.0f;
	else if(cam->angle[0] < -360.0f )
		cam->angle[0] += 360.0f;
	
	/* set temp quaternions */
	quat_init_axisangle( &qx, cam->angle[0], 1.0f, 0.0f, 0.0f);
	quat_init_axisangle( &qy, cam->angle[1], 0.0f, 1.0f, 0.0f);

	/* reset camera */
	quat_init_id( &(cam->rotation) );
	
	/* apply rotation */
	quat_mult( &qx, &qy);
	quat_mult( &(cam->rotation), &qx);
}

/*! \brief Move a camera */
void camera_move(camera_t *cam, GLfloat speed)
{
	vec_t new_pos;
	Uint32 sector_new;
	
	/* get direction */
	vec_t dir = quat_direction(&(cam->rotation));
	GLfloat motion = cam->motion * speed;
	
	/* move camera along direction */
	/* 
	cam->position.x += (dir.x * motion );
	cam->position.y += (dir.y * motion );
	cam->position.z += (dir.z * motion ); 
	*/

	new_pos.x = cam->position.x + (dir.x * motion );
	new_pos.y = cam->position.y + (dir.y * motion );
	new_pos.z = cam->position.z + (dir.z * motion );

	/* update current sector */
	sector_new = portal_traverse( hkw_ig->spg, &(cam->position), &new_pos);
	hkw_ig->spg->csector = sector_new;
	
	/* WARNING : blinking */
	/*
	hkw_ig->spg->csector = portal_traverse( hkw_ig->spg, &(cam->position), &new_pos);
	*/

	memcpy( &(cam->position), &new_pos, sizeof(vec_t));
}
