/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file frustum.c
 * \brief frustum culling
 */

#include "frustum.h"

/*! \brief extract view frustum planes */
void frustum_get( frustum_t *frustum )
{
	GLfloat m[16], mat_m[16], mat_p[16];

	glGetFloatv(GL_PROJECTION_MATRIX, mat_p);
	glGetFloatv(GL_MODELVIEW_MATRIX, mat_m);

	/* get GL_MODELVIEW_MATRIX * GL_PROJECTION_MATRIX */
	frustum_getmat( m, mat_p, mat_m );

	/* extract right && left plane */
	frustum_set_plane( &(frustum->fplane[0]), m[3] - m[0], m[7] - m[4], m[11] - m[8], m[15] - m[12] );
	frustum_set_plane( &(frustum->fplane[1]), m[3] + m[0], m[7] + m[4], m[11] + m[8], m[15] + m[12] );
	/* bottom, top */
	frustum_set_plane( &(frustum->fplane[2]), m[3] + m[1], m[7] + m[5], m[11] + m[9], m[15] + m[13] );
	frustum_set_plane( &(frustum->fplane[3]), m[3] - m[1], m[7] - m[5], m[11] - m[9], m[15] - m[13] );
	/* back, front */
	frustum_set_plane( &(frustum->fplane[4]), m[3] - m[2], m[7] - m[6], m[11] - m[10], m[15] - m[14] );
	frustum_set_plane( &(frustum->fplane[5]), m[3] + m[2], m[7] + m[6], m[11] + m[10], m[15] + m[14] );
	
	return;
}

/*! \brief set && normalize frustum plane */ 
void frustum_set_plane( plane_t *plane, float nvx, float nvy, float nvz, float pcst )
{
	GLfloat t = 0.0f;

	plane->nvec.x = nvx;
	plane->nvec.y = nvy;
	plane->nvec.z = nvz;

	/* normalize */
	t = vec_get_norm( &(plane->nvec) );

	plane->nvec.x /= t;
	plane->nvec.y /= t;
	plane->nvec.z /= t;
	plane->pcst = pcst / t;
}

/*! \brief multiply matrix_1 and matrix_2, result in mat_r */
void frustum_getmat( float *mat_r, float *mat_1, float *mat_2 )
{
	glPushMatrix();

	glLoadMatrixf(mat_1);
	glMultMatrixf(mat_2);
	
	glGetFloatv(GL_MODELVIEW_MATRIX, mat_r);

	glPopMatrix();
}
