/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file ingame.c
* \brief 'in game' functions
*/

#include "ingame.h"

/*! \brief initialize an 'in game' structure */
SDL_bool ig_init(Uint32 nmesh)
{
	SDL_Surface *main_screen = NULL;
	Uint32 i;

	if(hkw_ig == NULL)
	{
		hkw_ig = (ingame_t *)malloc(sizeof(ingame_t));
		if(hkw_ig == NULL)
			return SDL_FALSE;
		hkw_ig->cam_cur = NULL;
	}

	/* get screen dimension */

	main_screen = SDL_GetVideoSurface();

	if(main_screen == NULL)
	{
		return SDL_FALSE;
	}

	hkw_ig->width = main_screen->w;
	hkw_ig->height = main_screen->h;

	/* init camera */
	if( hkw_ig->cam_cur == NULL )
	{
		hkw_ig->cam_cur = (camera_t *)malloc(sizeof(camera_t));
		if( hkw_ig->cam_cur == NULL )
			return SDL_FALSE;
	}
	
	camera_init( hkw_ig->cam_cur );

	hkw_ig->nmesh = nmesh;

	hkw_ig->mesh_ig = (amesh_ig_t **)malloc(sizeof(amesh_ig_t *)*nmesh);

	if( hkw_ig->mesh_ig == NULL)
	{
		return SDL_FALSE;
	}
	else
	{
		for(i=0;i<nmesh;i++)
		{
			hkw_ig->mesh_ig[i] = NULL;
		}
	}

	hkw_ig->map = NULL;
	hkw_ig->spg = NULL;

	return SDL_TRUE;
}

/*! \brief free an 'in game' structure */
void ig_free()
{
	Uint32 i;

	if( hkw_ig == NULL )
		return;

	if(hkw_ig->cam_cur != NULL)
		free(hkw_ig->cam_cur);	

	if(hkw_ig->mesh_ig != NULL)
	{
		for(i=0;i<hkw_ig->nmesh;i++)
		{
			if(hkw_ig->mesh_ig[i] != NULL)
				amesh_ig_free(hkw_ig->mesh_ig[i]);
		}	
		free(hkw_ig->mesh_ig);
	}

	if(hkw_ig->map != NULL)
		map_free(hkw_ig->map);
	if(hkw_ig->spg != NULL)
		spg_free(hkw_ig->spg);
	if(hkw_ig != NULL)
		free(hkw_ig);
}

