/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file light.c
 * \brief light functions
 */

#include "light.h"

/*! \brief initialize and allocate a light with default values */
light_t *light_init()
{
	light_t *cur_light = NULL;

	cur_light = (light_t *)malloc(sizeof(light_t));

	if(cur_light == NULL)
	{
		return NULL;
	}
	else
	{
		
		cur_light->ambient[0] = 0.5f;
		cur_light->ambient[1] = 0.5f;
		cur_light->ambient[2] = 0.5f;
		cur_light->ambient[3] = 1.0f;
		
		cur_light->diffuse[0] = 1.0f;
		cur_light->diffuse[1] = 1.0f;
		cur_light->diffuse[2] = 1.0f;
		cur_light->diffuse[3] = 1.0f;

		cur_light->specular[0] = 1.0f;
		cur_light->specular[1] = 1.0f;
		cur_light->specular[2] = 1.0f;
		cur_light->specular[3] = 1.0f;
		
		cur_light->spot[0] = 0.0f;
		cur_light->spot[1] = 0.0f;
		cur_light->spot[2] = -1.0f;
		
		cur_light->position[0] = 0.0f;
		cur_light->position[1] = 2.0f;
		cur_light->position[2] = 0.0f;
		cur_light->position[3] = 1.0f;

		cur_light->att_const = 1.0f;
		cur_light->att_lin = 0.0f;
		cur_light->att_quad = 0.0f;
		
		cur_light->spot_cutoff = 180.0f;
		cur_light->spot_exponent = 0;
		return cur_light;
	}
}

/*! \brief used by map_load()
 * 
 * load lights from a map file
 */
light_t *light_load(FILE *map)
{
	char line[80];
	Uint32 data_section=0;
	int i=0;
	light_t *cur_light = NULL;
	char **split_data = NULL;
	
	cur_light = light_init();

	if(cur_light==NULL)
		return NULL;

	/*
	printf("map : loading lights\n");
	*/

	/* read number of vertices and faces */
	while(fgets(line, HKW_LINE_LENGTH, map))
	{
		if(isspace(line[0]))
			continue;
		if(line[0]==HKW_FILE_CHAR)
		{
			switch(line[1])
			{
				case 'l':
					data_section=0;
					break;
				case 's':
					/* light data begin */
					data_section=1;
					break;
				default:
				case 't':
					if(data_section!=9)
					{
						free(cur_light);
						cur_light = NULL;
					}
					return cur_light;
					break;
			}
		}

		i = string_split(line, &split_data, ' ');

		switch(data_section)
		{	
			case 2:
				if(i>=3)
				{
					cur_light->position[0] = atof(split_data[0]);
					cur_light->position[1] = atof(split_data[1]);
					cur_light->position[2] = atof(split_data[2]);
					cur_light->position[3] = atof(split_data[3]);
				}
				break;
			case 3:
				if(i>=3)
				{
					cur_light->spot[0] = atof(split_data[0]);
					cur_light->spot[1] = atof(split_data[1]);
					cur_light->spot[2] = atof(split_data[2]);
				}
				break;
			case 4:
				if(i>=3)
				{
					cur_light->ambient[0] = atof(split_data[0]);
					cur_light->ambient[1] = atof(split_data[1]);
					cur_light->ambient[2] = atof(split_data[2]);
				}	
				break;
			case 5:
				if(i>=3)
				{
					cur_light->diffuse[0] = atof(split_data[0]);
					cur_light->diffuse[1] = atof(split_data[1]);
					cur_light->diffuse[2] = atof(split_data[2]);
				}	
				break;
			case 6:
				if(i>=3)
				{
					cur_light->specular[0] = atof(split_data[0]);
					cur_light->specular[1] = atof(split_data[1]);
					cur_light->specular[2] = atof(split_data[2]);
				}	
				break;
			case 7:
				if(i>=3)
				{
					cur_light->att_const = atof(split_data[0]);
					cur_light->att_lin = atof(split_data[1]);
					cur_light->att_quad = atof(split_data[2]);
				}	
				break;
			case 8:
				if(i>=2)
				{
					cur_light->spot_cutoff = atof( split_data[0] ) ;
					cur_light->spot_exponent = (GLuint)atoi( split_data[1] ) ;
				}
			case 1:
			default:
				break;
		}

		data_section++;
		array_free(i, (void **)split_data);
	}
	
	free(cur_light);
	return NULL;
}

