/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file main.c
 * \brief main program entry
 */

#include "main.h"

/* proto */
void setup_opengl();
void setup_sdl();

sh_glsl_t sh0;
sh_glsl_t sh1;

/* argc && argv should be kept to compile on mingw32 */
int main(int argc, char **argv)
{
	fprintf(stdout, "Running hekaw version 0.0.5 ;-) : \n");
	
	setup_sdl();
	setup_opengl();

	/* initialize game objects : camera... */
	if(ig_init(3)==SDL_FALSE)
	{
		fprintf(stderr,"Could not initialize game objects : %s.\n", SDL_GetError());
		handle_exit();
	}

	/* Should be moved soon...*/
	if(!shader_init())
	{
		fprintf(stderr,"Could not initialize shader object : %s.\n", SDL_GetError());
		handle_exit();
	}

	if ( shader_create(&sh1, "data/shader/glsl_simple.vsh", "data/shader/glsl_ppl.fsh") == GL_FALSE )
		handle_exit();
	else
	{
		#if HKW_DEBUG_SHADER
		fprintf(stdout,"shader sh1 initialization done\n");
		#endif
	}

	hkw_ig->map = map_load("data/map/map_test.hkw");

	if(hkw_ig->map == NULL)
	{
		fprintf(stderr, "Could not load map : %s\n", SDL_GetError());
		handle_exit();
	}

	if( occlusion_init() == GL_FALSE )
	{
		fprintf(stderr, "Could not init occlusion, exiting...\n");
		handle_exit(); 	
	}

	hkw_ig->spg = spg_load("data/map/map_test.hkg", hkw_ig->map);

	if(hkw_ig->spg == NULL)
	{
		fprintf(stdout, "Could not load portal infos : %s\n", SDL_GetError());
		fprintf(stdout, "try to build a fake spg\n");
		hkw_ig->spg = spg_load_fake(hkw_ig->map);
		if( hkw_ig->spg == NULL )
		{
			fprintf(stderr, "Could not load portal infos : %s\n", SDL_GetError());
			handle_exit();
		}
	}

	hkw_ig->mesh_ig[0] = amesh_ig_init( "data/mesh/smesh_test.hka", SDL_FALSE, 0, 0);
	hkw_ig->mesh_ig[1] = amesh_ig_init( "data/mesh/amesh_test.hka", SDL_TRUE, 1, 2);
	hkw_ig->mesh_ig[2] = amesh_ig_init( "data/mesh/amesh_test.hka", SDL_TRUE, 0, 0);

	while(1)
	{
		process_events();
		update_screen();
		SDL_Delay(10);
	}

	/* should never happen */
	return SDL_FALSE;
}

/*! \brief setup opengl attributes */
void setup_opengl()
{
	SDL_Surface *main_screen = NULL;
	GLfloat ratio;

	fprintf(stdout, "OpenGL :\n");
	fprintf(stdout," RENDERER : %s\n", (char *)glGetString(GL_RENDERER));
	fprintf(stdout," VENDOR : %s\n", (char *)glGetString(GL_VENDOR));
	fprintf(stdout," VERSION : %s\n", (char *)glGetString(GL_VERSION));
	fprintf(stdout,"Glu :\n VERSION : %s\n", (char *)gluGetString(GLU_VERSION));

	main_screen = SDL_GetVideoSurface();

	if(main_screen == NULL)
	{
		fprintf(stderr,"Could not get main video surface : %s.\n", SDL_GetError());
		handle_exit();
	}

	ratio = (GLfloat) main_screen->w / (GLfloat) main_screen->h;	

	/* setup the viewport */
	glViewport( 0, 0, main_screen->w, main_screen->h );

	glMatrixMode( GL_PROJECTION ); 
	glLoadIdentity();

	/* setup correct and nicest perspective */
	gluPerspective( 45.0f, ratio, 0.1f, 50.0f );
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	/* set background color to black */
	glClearColor( 0.5f, 0.5f, 0.5f, 0.5f );

	glCullFace( GL_BACK );
	glFrontFace( GL_CCW );
	glEnable( GL_CULL_FACE );
	
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glEnable(GL_TEXTURE_2D); 
}

/*! \brief setup opengl attributes (via sdl) && sdl attributes
 *
 * Disable vsync and print some infos if HKW_DEBUG_SETUP = 1
 */
void setup_sdl()
{
	int width = 0, height = 0, bpp = 24, i=0;
	SDL_Rect **modes;
	int gl_attr;

	#if HKW_DEBUG_SETUP
	fprintf(stdout, "using SDL %d.%d.%d\n", SDL_MAJOR_VERSION, SDL_MINOR_VERSION, SDL_PATCHLEVEL);	
	#endif

	if((SDL_Init(SDL_INIT_VIDEO) <0)) 
	{
		fprintf(stderr,"Could not initialize sdl video : %s.\n", SDL_GetError());
		handle_exit();
	}

	/* search for available resolution on current hardware (&& select max res) */
	modes = SDL_ListModes(NULL, SDL_HWSURFACE|SDL_OPENGL|SDL_FULLSCREEN);

	/* from sdl doc : 'modes' are not guaranteed to be sorted in any particular order */
	if( modes == (SDL_Rect **)0 )
	{
		fprintf(stderr, "No modes available\n");
		handle_exit();
	}
	else
	{	
		for(i=0; modes[i]; i++)
		{
			if( width < modes[0]->w )
			{
				width = modes[i]->w;
				height = modes[i]->h;
			}
		}
	}

	#if HKW_DEBUG_SETUP
	for(i=0; modes[i]; i++)
	{
		fprintf(stdout, "available resolution %d : %d x %d\n", i, modes[i]->w, modes[i]->h);
	}	
	#endif

	fprintf(stdout, "display requested : %d x %d - %d\n", width, height, bpp);

	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, bpp );
	SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	/* set video mode */
	#if HKW_DEBUG_SETUP
	if( SDL_SetVideoMode(width, height, bpp, SDL_HWSURFACE|SDL_OPENGL) == 0 )
	#else
	if( SDL_SetVideoMode(width, height, bpp, SDL_HWSURFACE|SDL_OPENGL|SDL_FULLSCREEN) == 0 )
	#endif
	{
		fprintf(stderr,"Could not set required video mode: %s.\n", SDL_GetError());
		handle_exit();
	}
	
	fprintf(stdout, "display : %d %d %d --> ok\n", width, height, bpp);
	SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &gl_attr);
	fprintf(stdout, "double buffer : %d\n", gl_attr);
	SDL_GL_GetAttribute( SDL_GL_DEPTH_SIZE, &gl_attr );
	fprintf(stdout, "depth buffer size : %d\n", gl_attr);
	SDL_GL_GetAttribute( SDL_GL_STENCIL_SIZE, &gl_attr );
	fprintf(stdout, "stencil buffer size : %d\n", gl_attr);

	#if HKW_DEBUG_SETUP
		#if SDL_VERSION_ATLEAST(1, 2, 10)
		SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 0);
		SDL_GL_GetAttribute(SDL_GL_SWAP_CONTROL, &gl_attr);
		if(gl_attr==0)
		{
			fprintf(stdout, "disable vsync ok\n");
		}
		#endif
	#endif
}

