/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file mesh.h
 * \brief mesh header
 *
 * define a new type for mesh object : amesh_t
 */

#ifndef __defined_mesh_h
#define __defined_mesh_h

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>

#include <SDL_opengl.h>

#include "debug.h"
#include "c_util.h"
#include "extensions.h"
#include "skeleton.h"
#include "texture.h"
#include "shader.h"

/* define */
#define HKW_MESH_L1 "Hekaw smesh"
#define HKW_MESH_L1L 11
#define HKW_MESH_L2 "#blender"
#define HKW_MESH_L2L 8
#define HKW_MESH_NBFV 'i'
#define HKW_MESH_TEX 't'
#define HKW_MESH_SK 's'
#define HKW_MESH_ENDI 'd'
#define HKW_MESH_FACE 'f'
#define HKW_MESH_VERTICE 'v'
#define HKW_MESH_NORMAL 'n'
#define HKW_MESH_UV 'c'
#define HKW_MESH_ENDF 'e'
/* animated mesh define */
#define HKW_AMESH_L1 "Hekaw amesh"
#define HKW_AMESH_L1L 11
#define HKW_AMESH_WEIGHT 'w'
#define HKW_AMESH_UNAME1 "bm[%d]"
#define HKW_AMESH_ANAME1 "wei"
#define HKW_AMESH_ANAME2 "wei2"

/*! \brief amesh structure 
 *
 * this structure can handle both static and animated mesh
 */
typedef struct amesh_s
{
	Uint32 nbf;
	Uint32 nbv;
	/* Uint32 *faces; */
	GLuint *faces;
	GLfloat *vertices;
	GLfloat *normals;
	GLfloat *texcoord;
	GLuint *mat_idx;
	GLfloat *weights;
	GLuint texture[1];
	ask_t *skel;
}amesh_t;

/*! \brief amesh 'ingame' structure 
 *
 * this structure encapsulate an amesh structure
*/

typedef struct amesh_ig_s
{	
	Uint32 nact;
	int *act;
	Uint32 *cur_frame;

	amesh_t *mesh;
	Uint32 use_static;
	GLint *uniform_loc;
	GLint *attrib_loc;
	sh_glsl_t *cur_sh;
	sh_glsl_t *sh_sk;
	sh_glsl_t *sh_static;
}amesh_ig_t;

/* proto */
amesh_t *amesh_init(Uint32 nbf, Uint32 nbv, SDL_bool is_anim);
void amesh_free(amesh_t *mesh);

amesh_t *amesh_load(char *filename, SDL_bool is_anim);
amesh_t *amesh_load_d1(FILE *mesh, SDL_bool is_anim);
SDL_bool amesh_load_d2(FILE *mesh, amesh_t *cur_mesh);

amesh_ig_t *amesh_ig_init( char *fmesh, SDL_bool is_anim, Uint32 act1, Uint32 act2 );
void amesh_ig_free( amesh_ig_t *mesh_ig);

SDL_bool amesh_shader(amesh_ig_t *mesh_ig);
void amesh_display(amesh_ig_t *mesh_ig);

void amesh_update(amesh_ig_t *amesh, GLfloat u);

void amesh_update_shader1(amesh_ig_t *amesh, float u);
void amesh_update_shader2(amesh_ig_t *amesh, float u);

#endif

