/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info".
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file portal.c
 * \brief portal
 */

#include "portal.h"

extern sh_glsl_t sh1;

/*! \brief load a sector and portal graph */
spg_t *spg_load(char *filename, map_t *map)
{
	FILE *spg = NULL;
	spg_t *cur_spg = NULL;
	portal_t *cur_portal = NULL;
	sector_t *cur_sector = NULL;
	char line[HKW_LINE_LENGTH];
	Uint32 i = 0, j=0, k=0; /*, l=0; */
	/* Uint32 nlight = 0; */
	Uint32 rportal = 0, rsector = 0;
	/* char **split_data = NULL; */
	char ***portals = NULL;
	char ***sector_next = NULL;

	#if HKW_DEBUG_PORTAL
	fprintf(stdout, "loading portal informations : %s\n", filename);
	#endif

	spg = file_open(filename, "r+");

	if(!spg)
	{
		SDL_SetError("portal file not found...\n");
		return NULL;
	}

	/* read file header */
	fgets(line, HKW_LINE_LENGTH, spg);

	/* first line must be : Hekaw portal */
	if(strncmp(line+1, HKW_PORTAL_L1, HKW_PORTAL_L1L))
	{
		SDL_SetError("portal header is not correct\n");
		return NULL;
	}

	/* exporter used, mostly blender */
	fgets(line, HKW_LINE_LENGTH, spg);

	/* read number of objects & lights in file */
	while(fgets(line, HKW_LINE_LENGTH, spg))
	{
		if(isspace(line[0]))
			continue;
		if(line[0]==HKW_FILE_CHAR)
		{
			if(line[1]==HKW_PORTAL_NBO)
			{
				fgets(line, HKW_LINE_LENGTH, spg);
				i=atoi(line);
				fgets(line, HKW_LINE_LENGTH, spg);
				j=atoi(line);
				cur_spg = spg_init(i, j);
				/* printf("init spg with %u p and %u s\n", i, j); */

				/* */
				portals = (char ***)malloc(sizeof(char **)*j);

				for(i=0; i<j; i++)
					portals[i] = NULL;

				sector_next = (char ***)malloc(sizeof(char **)*j);

				for(i=0; i<j; i++)
					sector_next[i] = NULL;
				break;
			}
		}
	}

	/* free spg if NULL */
	if(cur_spg==NULL)
		return NULL;

	/* load portals */
	do
	{
		/* read portal */
		cur_portal = portal_load( spg, map, line);
		if( cur_portal != NULL )
		{
			/* assign portal to spg */
			cur_spg->portals[ rportal ] = cur_portal;
			cur_portal = NULL;
			/* read portal +1 */
			rportal++;
		}
		else
		{
			spg_free(cur_spg);
			return NULL;
		}
	}while( rportal < cur_spg->nbportal );

	/* load sectors */
	do
	{
		cur_sector = sector_load( spg, map, &(sector_next[rsector]), &(portals[rsector]), line);

		if( cur_sector != NULL )
		{
			cur_spg->sectors[ rsector ] = cur_sector;
			cur_sector = NULL;
			rsector++;
		}
		else
		{
			spg_free(cur_spg);
			return NULL;
		}
	}while( rsector < cur_spg->nbsector );

	/* resolve */
	rsector = 0;
	rportal = 0;
	for(i=0; i<cur_spg->nbsector; i++)
	{
		/* printf("proc. sector %d\n", i);
		printf("n portal for this sector : %d\n", cur_spg->sectors[i]->nbportal ); */
		for(j=0; j < cur_spg->sectors[i]->nbportal;j++)
		{
			/* printf("portal name : %s--\n", portals[i][j] ); */

			/* resolve portal name */
			for(k=0; k<cur_spg->nbportal; k++)
			{
				/* printf("- %s - vs - %s -\n", portals[i][j], cur_spg->portals[k]->name ); */
				if( strcmp( portals[i][j], cur_spg->portals[k]->name ) == 0 )
				{
					/* printf("--add portals to sector %d, p %d : %s --\n", i, j, cur_spg->portals[k]->name ); */
					cur_spg->sectors[i]->portals[j] = cur_spg->portals[k];
					rportal++;
				}
			}

			/* printf("next sector : %s\n", sector_next[i][j] ); */

			/* resolve sector name */
			for(k=0; k<cur_spg->nbsector; k++)
			{
				/* printf("- %s - vs - %s -\n", sector_next[i][j], cur_spg->sectors[k]->name ); */
				if( strcmp( sector_next[i][j], cur_spg->sectors[k]->name ) == 0 )
				{
					/* printf("--add next sector to sector %d, p %d : %s --\n", i, j, cur_spg->sectors[k]->name ); */
					cur_spg->sectors[i]->sectors[j] = k;
					rsector++;
				}
			}
		}

		if( rportal != cur_spg->sectors[i]->nbportal || rsector != cur_spg->sectors[i]->nbportal )
		{
			/* fprintf(stderr, "Could not resolve next sectors\n"); */
			SDL_SetError("could not link sector with its portals\n");
			spg_free(cur_spg);
			cur_spg = NULL;
			return NULL;
		}
		else
		{
			rportal = 0;
			rsector = 0;
		}
	}

	/* free arrays */
	for(i=0; i<cur_spg->nbsector; i++)
	{
		for(j=0; j<cur_spg->sectors[i]->nbportal; j++)
		{
			free(portals[i][j]);
			free(sector_next[i][j]);
		}
		free(portals[i]);
		free(sector_next[i]);
	}
	free(portals);
	free(sector_next);

	fclose(spg);

	/* */
	return cur_spg;
}

/*! \brief read portal section
 *
 * function called by spg_load
*/
portal_t *portal_load( FILE *spg, map_t *map, char *line )
{
	portal_t *portal = NULL;
	Uint32 i=0, j=0, k=0, l=0;
	char **split_data = NULL;
	
	if( spg == NULL || map == NULL )
		return NULL;
	
	/* read portal */
	while( fgets(line, HKW_LINE_LENGTH, spg) )
	{
		/* empty line - skip it */
		if(isspace(line[0]))
			continue;
		/* portal section ? */
		if(line[0]==HKW_FILE_CHAR)
		{
			if(line[1]==HKW_PORTAL_P)
			{
				l = string_split(line, &split_data, ' ');
				str_eol( split_data[1] );
				
				fgets(line, HKW_LINE_LENGTH, spg);
				i=atoi(line);
				fgets(line, HKW_LINE_LENGTH, spg);
				j=atoi(line);
				
				/* printf("%d faces && %d vertices\n", i, j); */
				portal = portal_init( i, j, split_data[1] );

				if( portal == NULL )
					return NULL;

				array_free(l, (void **)split_data);

				fgets(line, HKW_LINE_LENGTH, spg);

				for( k=0; k<i; k++)
				{
					fgets(line, HKW_LINE_LENGTH, spg);
					l = string_split(line, &split_data, ' ');
					if(l>=3)
					{
						portal->faces[ k*3 ] = atoi( split_data[0] );
						portal->faces[ k*3+1 ] = atoi( split_data[1] );
						portal->faces[ k*3+2 ] = atoi( split_data[2] );
					}
					array_free(l, (void **)split_data);
				}

				fgets(line, HKW_LINE_LENGTH, spg);

				for( k=0; k<j; k++)
				{
					fgets(line, HKW_LINE_LENGTH, spg);
					l = string_split(line, &split_data, ' ');
					if(l>=3)
					{
						portal->vertices[ k*3 ] = atof( split_data[0] );
						portal->vertices[ k*3+1 ] = atof( split_data[1] );
						portal->vertices[ k*3+2 ] = atof( split_data[2] );
					}
					array_free(l, (void **)split_data);
				}

				fgets(line, HKW_LINE_LENGTH, spg);

				for( k=0; k<i*3; k++)
				{
					fgets(line, HKW_LINE_LENGTH, spg);
					l = string_split(line, &split_data, ' ');
					if(l>=3)
					{
						portal->tris[ k*3 ] = atof( split_data[0] );
						portal->tris[ k*3+1 ] = atof( split_data[1] );
						portal->tris[ k*3+2 ] = atof( split_data[2] );
					}
					array_free(l, (void **)split_data);
				}

				/* plane */
				portal_plane( portal );
				break;

			}
		}
	}

	return portal;
}

/*! \brief read sector section
 *
 * function called by spg_load
*/
sector_t *sector_load( FILE *spg, map_t *map, char ***sector_next, char ***portals, char *line)
{
	Uint32 nlight = 0;
	sector_t *sector = NULL;
	char **split_data = NULL;
	Uint32 i=0, j=0, k=0, l=0;
	Uint32 nresolv=0;

	char **portals2 = NULL;
	char **sector_next2 = NULL;

	if( spg == NULL || map == NULL )
		return NULL;

	while( fgets(line, HKW_LINE_LENGTH, spg) )
	{
		if(isspace(line[0]))
			continue;
		if(line[0]==HKW_FILE_CHAR)
		{
			if(line[1]==HKW_PORTAL_S)
			{
				l = string_split(line, &split_data, ' ');
				str_eol( split_data[1] );

				fgets(line, HKW_LINE_LENGTH, spg);
				i=atoi(line);
				fgets(line, HKW_LINE_LENGTH, spg);
				j=atoi(line);
				fgets(line, HKW_LINE_LENGTH, spg);
				nlight=atoi(line);
				
				/* printf("%d obj - %d portals - %d lights\n", i, j, nlight); */
				
				sector = sector_init( i, j, nlight, split_data[1] );

				array_free(l, (void **)split_data);

				/* resolve mesh name */
				nresolv=0;
				for(k=0; k<i; k++)
				{
					fgets(line, HKW_LINE_LENGTH, spg);
					/* printf("add object : %s--\n", line ); */

					for(l=0; l<map->nsmesh;l++)
					{
						if( strncmp( line, map->meshname[l], strlen(map->meshname[l]) ) == 0 )
						{
							/* printf("--find obj idx: %d  in map: %s--\n", l, map->meshname[l]); */
							sector->mesh[k] = map->smesh[l];
							nresolv++;
						}
					}
				}

				/* check if we resolve all objects */
				if( nresolv != i )
				{
					/* fprintf(stderr, "could not resolve objects\n"); */
					SDL_SetError("could not resolve objects\n");
					sector_free(sector);
					return NULL;
				}

				portals2 = (char **)malloc(sizeof(char *)*j);
				sector_next2 = (char **)malloc(sizeof(char *)*j);
				
				for(k=0; k<j; k++)
				{
					fgets(line, HKW_LINE_LENGTH, spg);
					str_eol( line );
					portals2[k] = (char *)malloc(sizeof(char)*(strlen(line)+1));
					strncpy( portals2[k], line, strlen(line) );
					portals2[k][strlen(line)] = '\0';

					fgets(line, HKW_LINE_LENGTH, spg);
					str_eol( line );
					sector_next2[k] = (char *)malloc(sizeof(char)*(strlen(line)+1));
					strncpy( sector_next2[k], line, strlen(line) );
					sector_next2[k][strlen(line)] = '\0';
				}

				*portals = portals2;
				*sector_next = sector_next2;

				/* resolve light name */
				nresolv=0;
				for(i=0; i<nlight; i++)
				{
					fgets(line, HKW_LINE_LENGTH, spg);
					str_eol(line);
					/* printf("--> add light : %s\n", line ); */
					str_eol( line );
					for(l=0; l<map->nlight;l++)
					{
						/* printf("%s\n", map->lightname[l] ); */
						if( strncmp( line, map->lightname[l], strlen(map->lightname[l]) ) == 0 )
						{
							/* printf("--find light idx: %d  in map: %s--\n", l, map->lightname[l]); */
							/* sector->l_idx[i] = l; */
							sector->lights[i] = light_init();
							memcpy( sector->lights[i], map->light[l], sizeof(light_t));
							nresolv++;
						}
					}
				}

				/* check if we resolve all lights */
				if( nresolv != nlight )
				{
					/* fprintf(stderr, "could not resolve lights\n"); */
					SDL_SetError("could not resolve lights\n");
					sector_free(sector);
					return NULL;
				}

				break;
			}
		}
	}

	return sector;
}

/*! \brief create a spg with 1 sector and 0 portal */
spg_t *spg_load_fake(map_t *map)
{
	Uint32 i;
	spg_t *spg = NULL;
	sector_t *sector = NULL;

	spg = spg_init(0, 1);

	if( spg == NULL )
		return NULL;

	sector = sector_init( map->nsmesh, 0, map->nlight, "" );

	for(i=0; i<map->nsmesh; i++)
	{
		sector->mesh[i] = map->smesh[i];
	}

	/*
	for(i=0; i<map->nlight; i++)
	{
		sector->l_idx[i] = i;
	}
	*/
	
	for(i=0;i<map->nlight;i++)
	{
		sector->lights[i] = light_init();
		memcpy( sector->lights[i], map->light[i], sizeof(light_t) );
	}

	spg->sectors[0] = sector;
	sector = NULL;

	return spg;
}

/*! \brief init spg  */
spg_t *spg_init(Uint32 nportal, Uint32 nsector)
{
	Uint32 i;
	spg_t *spg = NULL;

	spg = (spg_t *)malloc(sizeof(spg_t));
	
	if( spg == NULL )
		return NULL;

	spg->nbportal = nportal;
	spg->nbsector = nsector;

	spg->portals = (portal_t **)malloc(sizeof(portal_t *)*nportal);
	spg->sectors = (sector_t **)malloc(sizeof(sector_t *)*nsector);
	spg->processed = (GLboolean *)calloc(nsector, sizeof(GLboolean));

	if( spg->portals == NULL || spg->sectors == NULL || spg->processed == NULL )
	{
		spg_free(spg);
		return NULL;
	}

	for(i=0; i<nportal; i++)
		spg->portals[i] = NULL;

	for(i=0; i<nsector; i++)
		spg->sectors[i] = NULL;

	spg->csector = 0;

	return spg;
}

portal_t *portal_init(Uint32 nf, Uint32 nv, char *name )
{
	Uint32 i=0;
	portal_t *portal = NULL;

	portal = (portal_t *)malloc(sizeof(portal_t));
	
	if( portal == NULL )
		return NULL;

	portal->nbf = nf;
	portal->nbv = nv;

	i = strlen( name );
	portal->name = (char *)malloc(sizeof(char)*(i+1));
	strncpy( portal->name, name, i );
	portal->name[i] = '\0';

	portal->faces = (Uint32 *)malloc(sizeof(Uint32)*nf*3);
	portal->vertices = (GLfloat *)malloc(sizeof(GLfloat)*nv*3);

	portal->tris = (GLfloat *)malloc(sizeof(GLfloat)*nf*9);

	if( portal->faces == NULL || portal->vertices == NULL )
	{
		portal_free(portal);
		return NULL;
	}
	else
	{
		return portal;
	}
}

sector_t *sector_init(Uint32 nmesh, Uint32 nportal, Uint32 nlight, char *name)
{
	Uint32 i=0;
	sector_t *sector = NULL;

	sector = (sector_t *)malloc(sizeof(sector_t));
	
	if( sector == NULL )
		return NULL;

	sector->nbmesh = nmesh;
	sector->nbportal = nportal;

	i = strlen( name );
	sector->name = (char *)malloc(sizeof(char)*i+1);
	strncpy( sector->name, name, i );
	sector->name[i] = '\0';

	sector->nlight = nlight;

	sector->mesh = (amesh_t **)malloc(sizeof(amesh_t *)*nmesh);
	sector->portals = (portal_t **)malloc(sizeof(portal_t *)*nportal);
	sector->sectors = (Uint32 *)malloc(sizeof(Uint32)*nportal);
	if( nlight > 0 )
	{
		/* sector->l_idx = (Uint32 *)malloc( sizeof(Uint32)*nlight ); */
		sector->lights = (light_t **)calloc(nlight, sizeof(light_t *));
		if( sector->lights == NULL )
		/* if( sector->l_idx == NULL ) */
		{
			sector_free(sector);
			return NULL;
		}
	}
	else
	{
		/* sector->l_idx = NULL; */
		sector->lights = NULL;
	}

	if( sector->mesh == NULL || sector->portals == NULL || sector->sectors == NULL )
	{
		sector_free(sector);
		return NULL;
	}
	else
	{
		return sector;
	}
}

/*! \brief free spg
 *
 * call sector_free && portal_free functions
 */
void spg_free( spg_t *cspg )
{
	Uint32 i;
	
	for(i=0; i<cspg->nbportal;i++)
	{
		portal_free( cspg->portals[i] );
		cspg->portals[i] = NULL;
	}
	free(cspg->portals);

	for(i=0; i<cspg->nbsector;i++)
	{
		sector_free( cspg->sectors[i] );
		cspg->sectors[i] = NULL;
	}
	free(cspg->sectors);

	free( cspg->processed );
	free( cspg );
}

void portal_free( portal_t *portal )
{
	if( portal != NULL )
	{
		if( portal->faces )
			free(portal->faces);
		if( portal->vertices )
			free(portal->vertices);
		if( portal->tris )
			free(portal->tris);
		if( portal->name )
			free(portal->name);
		free(portal);
	}
}

void sector_free( sector_t *sector )
{
	Uint32 i;

	if( sector != NULL )
	{
		/* map_free will free mesh */
		if( sector->mesh )
			free(sector->mesh);

		if( sector->portals )
			free(sector->portals);
		if( sector->sectors )
			free(sector->sectors);
		if( sector->name )
			free(sector->name);

		/*
		   if( sector->l_idx != NULL )
		   free(sector->l_idx);
		   */
		if( sector->lights != NULL )
		{
			for(i=0;i<sector->nlight;i++)
			{
				if( sector->lights[i] != NULL )
					free(sector->lights[i]);
			}
			free(sector->lights);
		}
		free(sector);
	}
}

void portal_plane( portal_t *portal )
{
	plane_t *plane = &(portal->plane);
	vec_t vec1, vec2;

	vec1.x = portal->vertices[3] - portal->vertices[0];
	vec1.y = portal->vertices[4] - portal->vertices[1];
	vec1.z = portal->vertices[5] - portal->vertices[2];

	vec2.x = portal->vertices[0] - portal->vertices[6];
	vec2.y = portal->vertices[1] - portal->vertices[7];
	vec2.z = portal->vertices[2] - portal->vertices[8];

	/* plane normal */
	plane->nvec = vec_cp( &vec1, &vec2 );
	vec_normalize( &(plane->nvec) );

	/* plane const */
	plane->pcst = -( portal->vertices[0] * plane->nvec.x + portal->vertices[1] * plane->nvec.y + portal->vertices[2] * plane->nvec.z );
}

/*! \brief render spg
 *
 * should be use with spg_display_begin && spg_display_end functions
*/
void spg_display( spg_t *cspg, int sec_idx )
{
	Uint32 nmesh, i;
	amesh_t *mesh;
	sector_t *csector = cspg->sectors[ sec_idx ];
	nmesh = csector->nbmesh;

	/* mark sector as displayed */
	if( cspg->processed[sec_idx] == GL_TRUE )
	{
		return;
	}
	else
	{
		cspg->processed[sec_idx] = GL_TRUE;
	}

	/* send light number to shader */
	/* _glUniform1iARB( _glGetUniformLocationARB( sh1.program, "nlight" ), csector->nlight ); */
	/* printf("n light : %d\n", csector->nlight ); */
	sector_light_set( csector );

	/* render */

	for(i=0; i<nmesh;i++)
	{
		mesh = csector->mesh[i];
		
		glNormalPointer(GL_FLOAT, 0, mesh->normals );
		glTexCoordPointer(2, GL_FLOAT, 0, mesh->texcoord );
		glVertexPointer(3, GL_FLOAT, 0, mesh->vertices );
		glBindTexture(GL_TEXTURE_2D, mesh->texture[0]);
		glDrawElements (GL_TRIANGLES, mesh->nbf * 3, GL_UNSIGNED_INT, mesh->faces );
	}

	for(i=0; i<csector->nbportal; i++)
	{
		if( portal_visible_f( cspg->portals[i] ) == GL_TRUE )
		{
			spg_display( cspg, csector->sectors[i] );
		}
	}
}

GLboolean occlusion_init()
{
	#if HKW_DEBUG_PORTAL
	GLint qbits;
	#endif

	/* need to check if ARB_occlusion_query is available on current hardware */
	if( !find_ext("GL_ARB_occlusion_query"))
	{
		SDL_SetError("GL_ARB_occlusion_query\n");
		return GL_FALSE;
	}
		
	_glBeginQueryARB = (PFNGLBEGINQUERYARBPROC)SDL_GL_GetProcAddress("glBeginQueryARB");	
	_glEndQueryARB = (PFNGLENDQUERYARBPROC)SDL_GL_GetProcAddress("glEndQueryARB");
	_glGetQueryObjectuivARB = (PFNGLGETQUERYOBJECTUIVARBPROC)SDL_GL_GetProcAddress("glGetQueryObjectuivARB");
	_glGenQueriesARB = (PFNGLGENQUERIESARBPROC)SDL_GL_GetProcAddress("glGenQueriesARB"); 
	_glGetQueryivARB = (PFNGLGETQUERYIVARBPROC)SDL_GL_GetProcAddress("glGetQueryivARB"); 

	if( (_glBeginQueryARB == NULL) || (_glEndQueryARB == NULL) || (_glGetQueryObjectuivARB == NULL) )
	{
		SDL_SetError("Couldn't get address of one or more functions of GL_ARB_occlusion_query\n");
		return GL_FALSE;
	}
	
	#if HKW_DEBUG_PORTAL
	fprintf(stdout, "GL_ARB_occlusion_query found\n");
	_glGetQueryivARB(GL_SAMPLES_PASSED_ARB, GL_QUERY_COUNTER_BITS_ARB, &qbits);
	fprintf(stdout, "Query counter bits = %d\n", qbits );
	#endif

	return GL_TRUE;
}

/*! \brief return GL_TRUE if portal is visible, GL_FALSE if unvisible
 *
 * use GL_ARB_occlusion_query extension
 */
GLboolean portal_visible_oc( portal_t *portal )
{
	GLuint oc_portal;
	GLuint samples = 0;
	
	glDepthMask( GL_FALSE );
	glColorMask( 0, 0, 0, 0 );
	
	glDisable( GL_CULL_FACE );

	_glGenQueriesARB(1, &oc_portal );	
	_glBeginQueryARB(GL_SAMPLES_PASSED_ARB, oc_portal);
	
	/*
	glVertexPointer(3, GL_FLOAT, 0, portal->vertices );
	glDrawElements (GL_TRIANGLES, portal->nbf * 3, GL_UNSIGNED_INT, portal->faces );
	*/

	glVertexPointer(3, GL_FLOAT, 0, portal->tris );
	glDrawArrays(GL_TRIANGLES, 0, portal->nbf*3 );
	
	_glEndQueryARB(GL_SAMPLES_PASSED_ARB);
		
	glDepthMask( GL_TRUE );
	glColorMask( 1, 1, 1, 1 );
	
	glEnable( GL_CULL_FACE );

	_glGetQueryObjectuivARB(oc_portal, GL_QUERY_RESULT_ARB, &samples);
	
	if( samples < 100 )
		return GL_FALSE;
	else
		return GL_TRUE;
}

/*! \brief return GL_TRUE if portal is visible, GL_FALSE if unvisible
 *
 * use frustum culling
 */
GLboolean portal_visible_f( portal_t *portal )
{
	GLboolean all_back;
	int i, j;
	frustum_t frustum;
	int nverts = portal->nbf*3;
	
	/* extract frustum */
	frustum_get( &frustum );

	/* test vertices against frustum planes */
	for(j=0; j<6; j++)
	{
		all_back = GL_TRUE;

		for(i=0; i < nverts; i++)
		{
			if( ( frustum.fplane[j].nvec.x * portal->tris[i*3] + frustum.fplane[j].nvec.y * portal->tris[i*3+1] + frustum.fplane[j].nvec.z * portal->tris[i*3+2] + frustum.fplane[j].pcst ) > 0 ) 
			{
				all_back = GL_FALSE;
				break;
			}
		}
		if(all_back)
		{
			return GL_FALSE;
		}
	}

	return GL_TRUE;
}

/*! \brief enable vertex array for spg rendering */
void spg_display_begin()
{
	glEnableClientState( GL_NORMAL_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	glEnableClientState( GL_VERTEX_ARRAY );
}

/*! \brief disable vertex array && reset displayed sector */
void spg_display_end( spg_t *cspg  )
{
	#if HKW_DEBUG_PORTAL
	Uint32 i, j=0;
	static Uint32 k=0;
	#endif

	glDisableClientState( GL_NORMAL_ARRAY );
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
	glDisableClientState( GL_VERTEX_ARRAY );

	#if HKW_DEBUG_PORTAL
	for(i=0; i<cspg->nbsector; i++)
	{
		if( cspg->processed[i] == GL_TRUE )
			j++;
	}

	if( j != k )
	{
		printf("displayed %u sectors\n", j);
		k = j;
	}
	#endif

	
	/* reset processed sectors */
	memset( cspg->processed, GL_FALSE, sizeof(GLboolean)*cspg->nbsector );
}

/*! \brief return new sector based on new position */
Uint32 portal_traverse( spg_t *cspg, vec_t *pos_old, vec_t *pos_new )
{
	Uint32 i;
	GLfloat dist_old, dist_new;
	portal_t *cportal;
	sector_t *csector = cspg->sectors[ cspg->csector ];

	/* for each portal in current sector */
	for(i=0; i<csector->nbportal; i++)
	{
		cportal = csector->portals[i];
		dist_old = plane_dist( &(cportal->plane), pos_old );
		dist_new = plane_dist( &(cportal->plane), pos_new );
		
		if( (dist_old * dist_new) < 0 )
		{
			if( ray_portal( cportal, pos_old, pos_new ) == SDL_TRUE )
			{
				#if HKW_DEBUG_PORTAL
				printf("traverse portal sector %u --> sector %u\n", i, csector->sectors[i]);
				#endif
				return csector->sectors[i];
			}
		}
	}

	return cspg->csector;
}

/*! \brief return SDL_TRUE if player collides a portal */
SDL_bool ray_portal( portal_t *portal, vec_t *pos_old, vec_t *pos_new )
{
	Uint32 i;
	vec_t pos_onplane;
	/* GLfloat test; */


	if( ray_plane( pos_old, pos_new, &(portal->plane), &pos_onplane ) == SDL_FALSE )
		return SDL_FALSE;

	for(i=0; i<portal->nbf;i++)
	{
		if( ray_in_tri2( portal->tris, &pos_onplane, i*9  ) == SDL_TRUE )
			return SDL_TRUE;
	}
	
	return SDL_FALSE;
}

/*! \brief render all sectors and portals in spg - for debugging purpose */
void spg_display_all( spg_t *cspg )
{
	int i, j;
	amesh_t *mesh;
	sector_t *csector = NULL; /* cspg->sectors[ sec_first ]; */
	portal_t *cportal;
	int nsec = cspg->nbsector;
	int nportal = cspg->nbportal;
	int nmesh = 0; /* csector->nbmesh; */

	glEnableClientState( GL_NORMAL_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	glEnableClientState( GL_VERTEX_ARRAY );

	for(i=0; i<nsec; i++)
	{
		/* printf("sector %d\n", i); */
		csector = cspg->sectors[ i ];
		nmesh = csector->nbmesh;
		
		for(j=0; j<nmesh; j++)
		{
			mesh = csector->mesh[j];
			
			glNormalPointer(GL_FLOAT, 0, mesh->normals );		
			glTexCoordPointer(2, GL_FLOAT, 0, mesh->texcoord );
			glVertexPointer(3, GL_FLOAT, 0, mesh->vertices );
			glBindTexture(GL_TEXTURE_2D, mesh->texture[0]);
			glDrawElements (GL_TRIANGLES, mesh->nbf * 3, GL_UNSIGNED_INT, mesh->faces );
		}
	}

	/* glEnable( GL_CULL_FACE ); */
	
	glDisableClientState( GL_NORMAL_ARRAY );
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
	glDisable( GL_CULL_FACE );
	_glUseProgramObjectARB(0);
	glColor3f( 1.0f, 0.0f, 0.0f );

	for(i=0; i<nportal; i++)
	{
		cportal = cspg->portals[ i ];
		
		glVertexPointer(3, GL_FLOAT, 0, cportal->vertices );
		glDrawElements (GL_TRIANGLES, cportal->nbf * 3, GL_UNSIGNED_INT, cportal->faces );
	}
	/* glDisable( GL_CULL_FACE ); */

	/*
	glDisableClientState( GL_NORMAL_ARRAY );
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
	*/
	glDisableClientState( GL_VERTEX_ARRAY );
}

/*! \brief set lights */
void sector_light_set( sector_t *sector )
{
	Uint32 i, j;
	light_t *light = NULL;

	#if HKW_DEBUG_LIGHT
	GLUquadric *quadratic;
	#endif

	j = HKW_MAP_MAXL < sector->nlight ? HKW_MAP_MAXL : sector->nlight;
	
	for(i=0; i<j; i++)
	{
		light = sector->lights[i];
		
		glLightfv(GL_LIGHT0+i, GL_POSITION, light->position );
		glLightfv(GL_LIGHT0+i, GL_SPOT_DIRECTION, light->spot);

		glLightfv(GL_LIGHT0+i, GL_AMBIENT, light->ambient);
		glLightfv(GL_LIGHT0+i, GL_DIFFUSE, light->diffuse);
		glLightfv(GL_LIGHT0+i, GL_SPECULAR, light->specular);

		glLightf( GL_LIGHT0+i, GL_CONSTANT_ATTENUATION, light->att_const ); 
		glLightf( GL_LIGHT0+i, GL_LINEAR_ATTENUATION , light->att_lin );
		glLightf( GL_LIGHT0+i, GL_QUADRATIC_ATTENUATION , light->att_quad );

		glLightf(GL_LIGHT0+i, GL_SPOT_CUTOFF, light->spot_cutoff);
		glLightf(GL_LIGHT0+i, GL_SPOT_EXPONENT, light->spot_exponent);
		
		/* display lights as red sphere with spot direction */
		#if HKW_DEBUG_LIGHT
		glTranslatef(light->position[0], light->position[1], light->position[2] );
		
		glDisable(GL_LIGHTING);
		
		glColor3f(1.0f, 0.0f, 0.0f);
		
		quadratic=gluNewQuadric();
		gluQuadricDrawStyle(quadratic, GL_FILL);
		gluQuadricNormals(quadratic, GLU_SMOOTH);
		gluSphere(quadratic, 0.1, 32, 32);
		
		glBegin(GL_LINES);
		glColor3f(1.0f, 0.0f, 0.0f);
		glVertex3f(0.0f, 0.0f, 0.0f);
		glColor3f(0.0f, 1.0f, 0.0f);
		
		glVertex3fv(light->spot);
		
		glEnd();
		
		glEnable(GL_LIGHTING);
		glTranslatef(-light->position[0], -light->position[1], -light->position[2] );
		
		gluDeleteQuadric(quadratic);
		#endif
	}
}

