/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use,
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info".
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability.
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or
* data to be ensured and,  more generally, to use and operate it in the
* same conditions as regards security.
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file portal.h
 * \brief portal header
 */

#ifndef __defined_portal_h
#define __defined_portal_h

/* standard C headers */
#include <stdlib.h>
#include <stdio.h>

#include <SDL.h>
#include <SDL_opengl.h>

#include "map.h"
#include "plane.h"
#include "collision.h"
#include "frustum.h"

#define HKW_PORTAL_L1 "Hekaw spg"
#define HKW_PORTAL_L1L 9
#define HKW_PORTAL_NBO 'm'
#define HKW_PORTAL_P 'p'
#define HKW_PORTAL_S 's'

typedef struct portal_s
{
	Uint32 nbf;
	Uint32 nbv;
	plane_t plane;
	GLuint *faces;
	GLfloat *vertices;
	GLfloat *tris;
	char *name;
}portal_t;

typedef struct sector_s
{
	Uint32 nbmesh;
	Uint32 nbportal;
	amesh_t **mesh;
	portal_t **portals;
	Uint32 *sectors;
	char *name;
	
	Uint32 nlight;
	/* light idx */
	/* Uint32 *l_idx; */
	light_t **lights;
}sector_t;

typedef struct spg_s
{
	Uint32 nbportal;
	Uint32 nbsector;
	portal_t **portals;
	sector_t **sectors;
	GLboolean *processed;
	Uint32 csector;
}spg_t;

/* proto */
spg_t *spg_load(char *filename, map_t *map);
portal_t *portal_load( FILE *spg, map_t *map, char *line );
sector_t *sector_load( FILE *spg, map_t *map, char ***sector_next, char ***portals, char *line);
spg_t *spg_load_fake(map_t *map);

spg_t *spg_init( Uint32 nportal, Uint32 nsector );
portal_t *portal_init( Uint32 nf, Uint32 nv, char *name );
sector_t *sector_init(Uint32 nmesh, Uint32 nportal, Uint32 nlight, char *name);

void spg_free( spg_t *cspg );
void portal_free( portal_t *portal );
void sector_free( sector_t *sector );

void spg_display( spg_t *cspg, int sec_idx );
void spg_display_begin();
void spg_display_end( spg_t *cspg  );
GLboolean occlusion_init();
GLboolean portal_visible_oc( portal_t *portal );
GLboolean portal_visible_f( portal_t *portal );

void portal_plane( portal_t *portal );
Uint32 portal_traverse( spg_t *cspg, vec_t *pos_old, vec_t *pos_new );
SDL_bool ray_portal( portal_t *portal, vec_t *pos_old, vec_t *pos_new );

void spg_display_all( spg_t *cspg );

void sector_light_set( sector_t *sector );

#endif
