/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file quaternion.h
 * \brief quaternion header
 *
 * define a new type for quaternion : quat_t
 */

#ifndef __defined_quaternion_h
#define __defined_quaternion_h

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <SDL.h>
#include <SDL_opengl.h>

#include "c_util.h"
#include "vector.h"

/*! \brief quaternion structure */
typedef struct quat_s
{
	GLfloat x, y, z, w;
}quat_t;

/* proto */
void quat_init_id(quat_t *q1);
void quat_init_axisangle( quat_t *quat, float theta, float x, float y, float z );
SDL_bool quat_isid( quat_t *quat );
void quat_mult( quat_t *q0, quat_t *q1 );
vec_t quat_direction(quat_t *q1);
void quat_tomat(quat_t *quat, GLfloat *matrix);
quat_t quat_slerp(quat_t *q1, quat_t *q2, float u);
quat_t quat_nlerp(quat_t *q1, quat_t *q2, float u);

#endif

