/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file screen.c
 * \brief screen functions
 */

#include "screen.h"

extern sh_glsl_t sh1;

/*! \brief update screen */
void update_screen()
{
	GLfloat cam_mat[16];
	camera_t *cam = hkw_ig->cam_cur;	

	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	quat_tomat( &(cam->rotation), cam_mat);
	/* glMultMatrixf( cam_mat ); */
	glLoadMatrixf( cam_mat );
	glTranslatef(cam->position.x, cam->position.y, cam->position.z);

	/* light_selection(hkw_ig->map); */
	
	_glUseProgramObjectARB(sh1.program);
	
	/*
	map_display(hkw_ig->map);
	*/
	
	spg_display_begin();
	spg_display( hkw_ig->spg, hkw_ig->spg->csector );
	spg_display_end( hkw_ig->spg );
	
	/*
	spg_display_all(hkw_ig->spg);
	*/

	_glUseProgramObjectARB(0);

	glTranslatef(-3.0f, 5.0f, -2.0f);
	
	amesh_display(hkw_ig->mesh_ig[1]);

	glTranslatef(6.0f, 0.0f, 2.0f);
		
	amesh_display(hkw_ig->mesh_ig[0]);

	glTranslatef(6.0f, 4.0f, 5.0f);

	amesh_display(hkw_ig->mesh_ig[2]);

	update_framerate();

	SDL_GL_SwapBuffers();
}

/*! \brief update animated mesh, camera and print fps
 * 
 * Update camera every 1/24s \n
 * Update animated mesh every 1/24s, interpolation every 1/60s \n
 * Print fps if #DEBUG=1 \n
 */
void update_framerate()
{
	#if HKW_DEBUG_SETUP
	static Uint32 fps_ltime = 0; 	
	#endif

	static Uint32 g_ltime = 0;
	Uint32 cur_time = SDL_GetTicks();
	Uint32 frame_interval = 0, i;

	hkw_ig->fps += 1;

	frame_interval = cur_time - g_ltime;

	/* update amesh every 1/24 s and camera */
	/* and interpolate every 1/60 s */
	if( frame_interval >= 42 )
	{	
		camera_move(hkw_ig->cam_cur, 0.3f);
		camera_mouse(hkw_ig->cam_cur);	
		g_ltime = cur_time;
		for(i=0; i< hkw_ig->nmesh; i++)
		{
			/* update hkw_ig->mesh_ig[i] */
			amesh_update( hkw_ig->mesh_ig[i], 1.0f ); 
		}
	}
	else if( frame_interval >= 14 )
	{
		for(i=0; i< hkw_ig->nmesh; i++)
		{
			/* update hkw_ig->mesh_ig[i]  */
			amesh_update( hkw_ig->mesh_ig[i], 14.0f/(float)frame_interval );
		}
	}	

	/* print fps */
	#if HKW_DEBUG_SETUP
	if( (cur_time - fps_ltime) >= 1000 )
	{

		fprintf(stdout, "fps : %u\n", hkw_ig->fps);

		hkw_ig->fps = 0;
		fps_ltime = cur_time;	
	}	
	#endif
}
