/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file skeleton.c
 * \brief skeleton functions
 */

#include "skeleton.h"

/*! \brief initialize a skeleton */
ask_t *ask_init(Uint32 nactions)
{
	Uint32 i;
	ask_t *cur_ask = NULL;

	cur_ask = (ask_t *)malloc(sizeof(ask_t));
	
	if(!cur_ask)
		return NULL;

	cur_ask->nactions = nactions;

	cur_ask->action = (sk_t **)malloc(sizeof(sk_t *)*nactions);

	if(!cur_ask)
		return NULL;

	for(i=0; i<nactions; i++)
	{
		cur_ask->action[i] = (sk_t *)malloc(sizeof(sk_t));
		if(!cur_ask->action[i])
		{
			ask_free(cur_ask);
			return NULL;
		}
	}
	
	return cur_ask;
}

/*! \brief free a skeleton */
void ask_free(ask_t *ask)
{
	Uint32 i;
	if(ask!=NULL)
	{
		if(ask->action != NULL)
		{
			for(i=0; i<ask->nactions; i++)
			{
				sk_free(ask->action[i]);
			}
			free(ask->action);
		}
		free(ask);
	}
}

/*! \brief load a skeleton */
ask_t *ask_load(char *filename)
{
	Uint32 i;
	FILE *sk = NULL;
	ask_t *cur_ask = NULL;
	char line[HKW_LINE_LENGTH];
	
	char *fpath = NULL;
	fpath = (char *)malloc( sizeof(char) * ( strlen(HKW_SKELETON_PATH(s)) + strlen(filename)) );

	if(fpath != NULL)
	{
		sprintf(fpath, HKW_SKELETON_PATH(s), filename);
		sk = file_open(fpath, "r+");
		free(fpath);
	}	
	else
		return NULL;

	if(!sk)
		return NULL;

	/* read first line of file */
	fgets(line, HKW_LINE_LENGTH, sk);

	/* first line must be : Hekaw skel */
	if(strncmp(line+1, HKW_HEADER_SKL1, HKW_HEADER_SKL1L))
		return NULL;

	/* exporter used, mostly blender */
	fgets(line, HKW_LINE_LENGTH, sk);

	/* gets number of actions */
	while(fgets(line, HKW_LINE_LENGTH, sk))
	{
		if(isspace(line[0]))
			continue;
		if(line[0]==HKW_FILE_CHAR)
		{
			if(line[1]==HKW_SK_NBACTION)
			{
				fgets(line, HKW_LINE_LENGTH, sk);
				i = atoi(line);
				cur_ask = ask_init( i );
				break;
			}
		}
	}

	if(!cur_ask)
	{
		fclose(sk);
		return NULL;
	}

	for(i=0; i < cur_ask->nactions; i++)
	{
		cur_ask->action[i] = sk_load_data2(sk);
		if(cur_ask->action[i] == NULL)
		{
			ask_free(cur_ask);
			cur_ask=NULL;
			break;
		}
	}
	
	fclose(sk);
	return cur_ask;
}

sk_t *sk_init(Uint32 nbones, Uint32 nframes)
{
	sk_t *cur_sk = NULL;
	Uint32 i;

	cur_sk = (sk_t *)malloc(sizeof(sk_t));

	if(cur_sk)
	{
		cur_sk->nbones=nbones;
		cur_sk->nframes=nframes;
		cur_sk->quat = (quat_t **)malloc(sizeof(quat_t *)*nframes);
		
		for(i=0; i<nframes; i++)
		{
			(cur_sk->quat)[i]=(quat_t *)malloc(sizeof(quat_t)*nbones);
		}

		cur_sk->trans = (vec_t **)malloc(sizeof(vec_t *)*nframes);
		for(i=0; i<nframes; i++)
		{
			(cur_sk->trans)[i]=(vec_t *)malloc(sizeof(vec_t)*nbones);
		}
	}

	return cur_sk;
}

void sk_free(sk_t *sk)
{
	Uint32 i;
	if(sk != NULL)
	{
		if( sk->quat != NULL )
		{
			for(i=0; i< sk->nframes; i++)
			{
				if( sk->quat[i] != NULL)
					free(sk->quat[i]);
			}
			free(sk->quat);
		}
		if( sk->trans != NULL )
		{
			for(i=0; i< sk->nframes; i++)
			{
				if( sk->trans[i] != NULL)
					free(sk->trans[i]);
			}
			free(sk->trans);
		}
		free(sk);
	}
}

/*
 * load a skeleton
 */
sk_t *sk_load_data2(FILE *sk)
{
	sk_t *cur_sk = NULL;
	char **split_data = NULL;
	int i;
	Uint32 nbf = 0, nbo = 0;
	Uint32 cur_frame=0, cur_bo=0,data_section=0;
	SDL_bool cur_data = SDL_TRUE;
	char line[80];
	
	/* read number of bones and frames then init */
	while(fgets(line, HKW_LINE_LENGTH, sk))
	{
		if(isspace(line[0]))
			continue;
		if(line[0]==HKW_FILE_CHAR)
		{
			if(line[1]==HKW_SK_NBOF)
			{
				fgets(line, HKW_LINE_LENGTH, sk);
				nbo=atoi(line);
				fgets(line, HKW_LINE_LENGTH, sk);
				nbf=atoi(line);
				cur_sk = sk_init(nbo, nbf);
				break;
			}
		}
	}
	
	if(cur_sk==NULL)
		return NULL;

	/* read data */
	while(fgets(line, HKW_LINE_LENGTH, sk))
	{
		if(isspace(line[0]))
			continue;
		if(line[0]==HKW_FILE_CHAR)
		{
			switch(line[1])
			{
				/* start of a new frame */
				case 'f':
					data_section=1;
					break;
				/* end of a frame */
				case 'g':
					data_section=0;
					cur_frame++;
					/* test if we read enough bone */
					if( cur_bo != nbo )
					{
						sk_free(cur_sk);
						return NULL;
					}
					else
						cur_bo=0;
					break;
				/* end of an action */
				default:
				case 'b':
					/* test if we read enough frame */
					if( cur_frame!=nbf )
					{
						sk_free(cur_sk);
						cur_sk=NULL;
					}
					return cur_sk;
					break;
			}
			continue;
		}

		switch(data_section)
		{
			case 1:
				i = string_split(line, &split_data, ' ');

				if((cur_data==SDL_FALSE) && (i>=3))
				{
					(cur_sk->trans)[cur_frame][cur_bo].x=atof(split_data[0]);
					(cur_sk->trans)[cur_frame][cur_bo].y=atof(split_data[1]);
					(cur_sk->trans)[cur_frame][cur_bo].z=atof(split_data[2]);
					/* next bone */
					cur_bo++;
					/* fill quaternion next time */
					cur_data=SDL_TRUE;
				}
				else if((cur_data==SDL_TRUE) && (i>=4))
				{
					(cur_sk->quat)[cur_frame][cur_bo].x=atof(split_data[0]);
					(cur_sk->quat)[cur_frame][cur_bo].y=atof(split_data[1]);
					(cur_sk->quat)[cur_frame][cur_bo].z=atof(split_data[2]);
					(cur_sk->quat)[cur_frame][cur_bo].w=atof(split_data[3]);
					/* fill translation vector next time */
					cur_data=SDL_FALSE;
				}
				else
				{
					/* there is an error in the file here --> reset cur_bo */
					cur_bo=0;
				}
				/*
				if(cur_bo>nbo)
					cur_bo=0; */

				array_free(i, (void **)split_data );
				
				break;
			default:
				break;
		}
	}
	
	/*  reach end of file */
	sk_free(cur_sk);
	return NULL;
}
