/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file skeleton.h
 * \brief skeleton header
 */

#ifndef __defined_skeleton_h
#define __defined_skeleton_h

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <ctype.h>

#include <SDL_opengl.h>

#include "c_util.h"
#include "quaternion.h"
#include "vector.h"

/* define */
#define HKW_HEADER_SKL1 "Hekaw skel"
#define HKW_HEADER_SKL1L 10
#define HKW_SK_NBACTION 's'
#define HKW_SK_NBOF 'i'

#define HKW_SKELETON_PATH(s) "data/skeleton/%s"

/*! \brief used by skeleton structure
 *
 * represent a skeleton with one action
*/
typedef struct sk_s
{
	Uint32 nbones;
	Uint32 nframes;
	quat_t **quat;
	vec_t **trans;
}sk_t;

/*! \brief skeleton structure
 * 
 * this structure is basically a collection of sk structure
 */
typedef struct ask_s
{
	Uint32 nactions;
	sk_t **action;	
}ask_t;

ask_t *ask_init(Uint32 nactions);
void ask_free( ask_t *ask);
ask_t *ask_load(char *filename);

sk_t *sk_init(Uint32 nbones, Uint32 nframes);
void sk_free(sk_t *sk);
sk_t *sk_load_data(FILE *sk);

sk_t *sk_load_data2(FILE *sk);

#endif

