/*
* Copyright dylhoxic
*
* dylhoxic _at_ tuxfamily _dot_ org
*
* This software is a first person shooter game.
*
* This software is governed by the CeCILL version 2 license under French law and
* abiding by the rules of distribution of free software.  You can  use, 
* modify and/ or redistribute the software under the terms of the CeCILL
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info". 
*
* As a counterpart to the access to the source code and rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability. 
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or 
* data to be ensured and,  more generally, to use and operate it in the 
* same conditions as regards security. 
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL version 2 license and that you accept its terms.
*/

/*! \file texture.c
 * \brief texture functions
 */

#include "texture.h"

/*! \brief basic bmp loader */
SDL_bool texture_load(char *filename, GLuint *tex_index)
{
	SDL_Surface *texture = NULL;
	SDL_Surface *texture_flip = NULL;
	int i,j;
	Uint32 rmask, gmask, bmask;
	char *fpath = NULL;
	
	/* 24 bits masks ! */
	#if SDL_BYTEORDER == SDL_BIG_ENDIAN
		rmask = 0x0000ff;
		gmask = 0x00ff00;
		bmask = 0xff0000;
	#else
		rmask = 0xff0000;
		gmask = 0x00ff00;
		bmask = 0x0000ff;
	#endif	

	fpath = (char *)malloc( sizeof(char) * ( strlen(HKW_TEXTURE_PATH(s)) + strlen(filename)) );
	if(fpath != NULL)
	{
		sprintf(fpath, HKW_TEXTURE_PATH(s), filename);
		
		#if HKW_DEBUG_TEX
		fprintf(stdout, "loading texture : %s\n", fpath);
		#endif
		
		texture = SDL_LoadBMP(fpath);
		free(fpath);
	}
	else
	{
		return SDL_FALSE;
	}

	if( texture == NULL)
	{
		return SDL_FALSE;
	}

	texture_flip = SDL_CreateRGBSurface(SDL_SWSURFACE, texture->w, texture->h
			, texture->format->BitsPerPixel,
			rmask, gmask, bmask, 0);

	if(texture_flip == NULL)
	{
		/* SDL_SetError("Could not create flipped surface\n"); */
		return SDL_FALSE;
	}	
	
	if(texture->format->palette != NULL)
	{
		SDL_SetError("Textures with palette are not supported\n");
		return SDL_FALSE;
	}
	else 
	{	
		SDL_LockSurface(texture);
		SDL_LockSurface(texture_flip); 
		for( i=0 ; i< texture->h ; i++)
		{
			for( j=0 ; j< texture->w ; j++)
			{
				putpixel(texture_flip, i, texture->h - j - 1, getpixel(texture, i, j));
			}
		}
		SDL_UnlockSurface(texture);
		SDL_UnlockSurface(texture_flip);

	}

	glGenTextures(1, tex_index);
	glBindTexture(GL_TEXTURE_2D, *tex_index);
	
	glTexParameteri	(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
	glTexParameteri	(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	gluBuild2DMipmaps(GL_TEXTURE_2D, texture_flip->format->BytesPerPixel, texture_flip->w, 
			texture_flip->h, GL_BGR, GL_UNSIGNED_BYTE, texture_flip->pixels);

	SDL_FreeSurface(texture);
	SDL_FreeSurface(texture_flip);
	return SDL_TRUE;
}

